/*-
 * ~~~~~~licensing~~~~~~
 * test-producers
 * ==========
 * Copyright (C) 2020 - 2022 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.base.objects.shell;

import java.util.List;
import java.util.stream.Collectors;

import org.apache.karaf.shell.api.action.Action;
import org.apache.karaf.shell.api.action.Argument;
import org.apache.karaf.shell.api.action.Completion;

import ru.entaxy.platform.base.objects.EntaxyObject;
import ru.entaxy.platform.base.objects.shell.completers.ObjectCompleter;

public abstract class ObjectAwareCommand extends EntaxyObjectServiceSupport implements Action {
	
	@Argument (index = 0, name = "objectFullId", required = true)
	@Completion(caseSensitive = false, value = ObjectCompleter.class)
	String objectFullId;

	EntaxyObject entaxyObject;
	
	@Override
	public Object execute() throws Exception {
		
		List<EntaxyObject> candidates = objectService.getObjects()
				.stream().filter(obj->obj.getObjectFullId().equals(objectFullId))
				.collect(Collectors.toList());
		if (candidates.size()>0) {
			entaxyObject = candidates.get(0);
			return doExecute();
		} else {
			throw new IllegalArgumentException("Object not found by full Id: [" + objectFullId + "]");
		}
	}

	public abstract Object doExecute() throws Exception;
	
}
