/*-
 * ~~~~~~licensing~~~~~~
 * test-producers
 * ==========
 * Copyright (C) 2020 - 2022 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.base.objects.storage.impl;

import java.util.HashMap;
import java.util.Map;

import org.osgi.service.component.annotations.CollectionType;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.osgi.service.component.annotations.ReferencePolicy;
import org.osgi.service.component.annotations.ReferencePolicyOption;

import ru.entaxy.platform.base.objects.EntaxyObjectStorage;
import ru.entaxy.platform.base.objects.EntaxyObjectStorageService;

@Component(service = EntaxyObjectStorageService.class, immediate = true)
public class EntaxyObjectStorageServiceImpl implements EntaxyObjectStorageService {

	protected Map<String, EntaxyObjectStorage> objectStorages = new HashMap<>();
	
	@Override
	public EntaxyObjectStorage getObjectStorage(String objectType) {
		return this.objectStorages.get(objectType);
	}

	@Reference(bind = "addObjectStorage", unbind = "removeObjectStorage", cardinality = ReferenceCardinality.MULTIPLE
			, policy = ReferencePolicy.DYNAMIC, policyOption = ReferencePolicyOption.GREEDY, collectionType = CollectionType.SERVICE)
	public void addObjectStorage(EntaxyObjectStorage objectStorage) {
		String[] types = objectStorage.getObjectTypes();
		if (types == null)
			return;
		for (int i=0; i<types.length; i++)
			this.objectStorages.put(types[i], objectStorage);
	}
	
	public void removeObjectStorage(EntaxyObjectStorage objectStorage) {
		String[] types = objectStorage.getObjectTypes();
		if (types == null)
			return;
		for (int i=0; i<types.length; i++)
			this.objectStorages.remove(types[i]);
	}
	
}
