/*-
 * ~~~~~~licensing~~~~~~
 * test-producers
 * ==========
 * Copyright (C) 2020 - 2022 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.producer.executor.generationmodel;

import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import ru.entaxy.esb.platform.runtime.base.connecting.generator.Generated;
import ru.entaxy.platform.core.producer.api.EntaxyProducerService;
import ru.entaxy.platform.core.producer.api.ProducerResult;
import ru.entaxy.platform.core.producer.api.ProducerResult.CommandResult;

public abstract class AbstractTask {

	protected boolean isPrintOutput = false;
	
	protected List<Generated> oldResults = new LinkedList<>();
	
	protected Generated generatedResult = null;
	
	protected AbstractTask mergeTo = null;
	
	protected boolean isFinal = true;
	
	public abstract String getInfo();

	public abstract boolean execute(
			ProducerResult currentResult
			, CommandResult currentCommand
			, EntaxyProducerService producerService
			, Map<String, Object> instructions) throws Exception;
	
	protected void printOutput(String message) {
		if (this.isPrintOutput)
			// OUTPUT TO CONSOLE
			System.out.println(message);
	}
	
	public boolean isPrintOutput() {
		return isPrintOutput;
	}

	public void setPrintOutput(boolean isPrintOutput) {
		this.isPrintOutput = isPrintOutput;
	}

	public Generated getGeneratedResult() {
		return generatedResult;
	}

	public void mergeTo(AbstractTask abstractTask) {
		this.mergeTo = abstractTask;
	}
	
	public boolean isToMerge() {
		return this.mergeTo != null;
	}

	public boolean isFinal() {
		return isFinal;
	}

	public void setFinal(boolean isFinal) {
		this.isFinal = isFinal;
	}
	
	public void updateGenerated(Generated generated) {
		oldResults.add(generatedResult);
		this.generatedResult = generated;
	}
}
