/*-
 * ~~~~~~licensing~~~~~~
 * object-producer-core
 * ==========
 * Copyright (C) 2020 - 2022 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.producer.executor.generationmodel;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.google.gson.Gson;

import ru.entaxy.platform.base.support.JSONUtils;
import ru.entaxy.platform.core.producer.api.EntaxyProducerService;
import ru.entaxy.platform.core.producer.api.ProducerResult;
import ru.entaxy.platform.core.producer.api.ProducerResult.CommandResult;
import ru.entaxy.platform.core.producer.executor.objectmodel.Calculation;
import ru.entaxy.platform.core.producer.executor.objectmodel.FactoredObject;
import ru.entaxy.platform.core.producer.executor.objectmodel.ObjectModel;

public class CalculateTask extends AbstractObjectTask {

	private static final Logger log = LoggerFactory.getLogger(CalculateTask.class);

	public static boolean hasCalculations(FactoredObject factoredObject) {
		return factoredObject.getInternals().stream().filter(obj -> (obj instanceof Calculation))
				.count() > 0;
	}
	
	public CalculateTask(FactoredObject factoredObject) {
		super(factoredObject);
	}

	@Override
	public String getInfo() {
		return "CALCULATE FOR " + "[" + origin.getObjectId() + "]";
	}

	@Override
	public boolean execute(ProducerResult currentResult, CommandResult currentCommand,
			EntaxyProducerService producerService, Map<String, Object> instructions) throws Exception {
		
		ObjectModel objectModel = currentResult.findResultObject(ObjectModel.class);
		
		List<Calculation> calculations = this.origin.getInternals().stream()
				.filter(obj -> (obj instanceof Calculation))
				.map(obj -> (Calculation)obj)
				.collect(Collectors.toList());
		
		for (Calculation calculation: calculations) {
			Object result = calculation.calculate(objectModel, origin);
			System.out.println("CALCULATION RESOLVED TO :: " + result);
			System.out.println("\n REPLACE"
					+ "\n\t" + calculation.origin
					+ "\n\tWITH :: " + (new Gson()).toJsonTree(result)
					+ "\n\tAT :: " + calculation.relativePath);
			JSONUtils.replaceValue(origin.origin, calculation.relativePath, (new Gson()).toJsonTree(result));

		}
		
		return true;
	}

}
