/*-
 * ~~~~~~licensing~~~~~~
 * object-producing-management
 * ==========
 * Copyright (C) 2020 - 2022 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.producer.management.impl;

import java.util.Iterator;

import javax.management.NotCompliantMBeanException;

import org.osgi.framework.BundleContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.esb.platform.base.management.core.utils.ManagedObjectsListener;
import ru.entaxy.platform.core.producer.management.ProducerMBean;

public class TypedProducerListener extends ManagedObjectsListener<ManagedTypedProducers, ManagedTypedProducer, ProducerMBean, ProducerMBeanImpl> {

	private static final Logger log = LoggerFactory.getLogger(TypedProducerListener.class);
	
	public TypedProducerListener(BundleContext bundleContext, Class<ProducerMBean> serviceClass) {
		super(bundleContext, serviceClass);
	}

	@Override
	protected Iterator<ManagedTypedProducer> getIterator(ManagedTypedProducers managedObject) {
		return managedObject.getProducers().iterator();
	}

	@Override
	protected String getKeyValue(ManagedTypedProducer object) {
		return object.supportedType;
	}

	@Override
	protected ProducerMBeanImpl createService(ManagedTypedProducer managedObject) {
		try {
			return new ProducerMBeanImpl(this.bundleContext, managedObject.supportedType, managedObject.producer);
		} catch (NotCompliantMBeanException e) {
			log.error("Error creating ProducerMBeanImpl: ", e);
			return null;
		}
	}

	@Override
	protected String getJmxObjectName(ManagedTypedProducer managedObject) {
		return ProducerMBean.getQualifier(managedObject.supportedType).getValue();
	}

	@Override
	public void removed(ManagedTypedProducers managedObject) {
		for (ManagedTypedProducer mtp: managedObject.getProducers()) {
			getManaged()
				.get(getKeyValue(mtp))
				.service
				.beforeRemove();
		}
		super.removed(managedObject);
	}
	
}
