/*-
 * ~~~~~~licensing~~~~~~
 * test-producers
 * ==========
 * Copyright (C) 2020 - 2022 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.base.objects.profile.storage;

import com.google.gson.Gson;
import com.google.gson.JsonObject;
import org.osgi.service.component.annotations.CollectionType;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import ru.entaxy.esb.system.jpa.SystemService;
import ru.entaxy.esb.system.management.bundle.jpa.entity.BundleEntity;
import ru.entaxy.platform.base.objects.EntaxyObject;
import ru.entaxy.platform.base.objects.EntaxyObjectStorage;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

@Component(service = EntaxyObjectStorage.class, immediate = true)
public class SystemObjectStorage implements EntaxyObjectStorage {

    private static final Logger log = LoggerFactory.getLogger(SystemObjectStorage.class);

    @Reference(cardinality = ReferenceCardinality.MANDATORY, collectionType = CollectionType.SERVICE)
    protected SystemService systemService;

    @SuppressWarnings("serial")
    protected Map<String, String> fieldMap = new HashMap<>() {{
        put("uuid", "objectId");
        put("name", "systemName");
        put("description", "description");
        put("version", "artifact.version");
        put("isDefaultRoute", "useDefaultRoute");
    }};

    protected Map<String, String> fieldBundleMap = new HashMap<>() {{
        put("name", "bundleId");
        put("type", "artifact.type");
        put("status", "artifact.status");
        put("version", "artifact.version");
        put("url", "bundleLocation");
    }};

    @Override
    public String[] getObjectTypes() {
        return new String[]{EntaxyObject.OBJECT_TYPES.PROFILE};
    }

    @Override
    public Object store(Map<String, Object> data) {

        JsonObject jsonObject = new JsonObject();
        Gson gson = new Gson();

        for (Entry<String, String> entry : fieldMap.entrySet()) {
            if (data.containsKey(entry.getValue()))
                jsonObject.add(entry.getKey(), gson.toJsonTree(data.get(entry.getValue())));
        }
        JsonObject bundleJsonObject = new JsonObject();
        for (Entry<String, String> entry : fieldBundleMap.entrySet()) {
            if (data.containsKey(entry.getValue()))
                bundleJsonObject.add(entry.getKey(), gson.toJsonTree(data.get(entry.getValue())));
        }

        log.debug("JSON :: " + jsonObject.toString());
        ru.entaxy.esb.system.jpa.entity.System system = gson.fromJson(jsonObject, ru.entaxy.esb.system.jpa.entity.System.class);
        system.setCreateDate(new Date());
        system.setCreatedBy("admin");

        log.debug("JSON Bundle :: " + bundleJsonObject.toString());
        BundleEntity bundleEntity = gson.fromJson(bundleJsonObject, BundleEntity.class);
        system.setBundleEntity(bundleEntity);
        systemService.create(system);

        return null;
    }

}
